<?php

declare(strict_types=1);

namespace hood\vk\api;

use hood\vk\robots\robot;

use Throwable;
use Exception;
use ArrayAccess;
use GuzzleHttp\Promise\Each;

/**
 * API ВКонтакте
 *
 * @var robot $robot Робот
 * @var array $settings Настройки
 * @var float $version Версия API
 */
class api implements ArrayAccess
{
    /**
     * Конструктор
     */
    public function __construct(
        protected robot $robot,
        protected array $settings = [],
        protected float $version = 5.124
    ) {
        if (empty($settings)) {
            // Настройки не получены

            // Инициализация
            $this->_init();
        }
    }

    /**
     * Инициализация (безопасная)
     */
    public function init(): array
    {
        if (!isset($blocked)) {
            // Нет блокировки (запуск в первый раз)

            // Блокировка
            static $blocked = true;

            try {
                $this->_init();
            } catch (Throwable $t) {
                throw new Exception('Не удалось инициализировать API', $t->getCode(), $t->getPrevious());
            }
        }

        return $this->settings;
    }

    /**
     * Реинициализация
     */
    public function reinit(): array
    {
        // Реинициализация
        //unset($this->settings);
        $this->settings = [];

        try {
            $this->_init();
        } catch (Throwable $t) {
            throw new Exception('Не удалось инициализировать API', $t->getCode(), $t->getPrevious());
        }

        return $this->settings;
    }

    /**
     * Инициализация
     */
    protected function _init(): void
    {
        // Ключ
        $this->settings['access_token'] = $this->robot->key;

        // Версия API
        $this->settings['v'] = $this->version;
    }

    /**
     * Выбрать получателя
     *
     * Определяет получателей по входным параметрам
     *
     * @see hood\vk\api\methods\messages Сообщения
     */
    public function chooseDestination(string|array|int $destination): void
    {
        if (is_int($destination)) {
            // Идентификатор

            $this->settings['peer_id'] = $destination;
        } else if (is_array($destination)) {
            // Идентификаторы

            $this->settings['user_ids'] = $destination;
        } else {
            // Домен

            $this->settings['domain'] = $destination;
        }
    }

    /**
     * Записать свойство
     *
     * @param string $name Название
     * @param mixed $value Значение
     */
    public function __set(string $name, mixed $value): void
    {
        match ($name) {
            'robot' => !isset($this->robot) ? $this->robot = $value : throw new Exception('Запрещено перезаписывать робота'),
            'settings' => !isset($this->settings) ? $this->settings = $value : throw new Exception('Запрещено перезаписывать настройки'),
            default => $this->settings[$name] = $value
        };
    }

    /**
     * Прочитать свойство
     *
     * @param string $name Название
     */
    public function __get(string $name): mixed
    {
        return match ($name) {
            'robot' => $this->robot ?? throw new Exception('Робот не инициализирован'),
            'settings' => $this->settings ?? throw new Exception('Настройки не инициализированы'),
            default => $this->settings[$name] ?? throw new Exception('Параметр не найден: settings[\'' . $name . '\']')
        };
    }

    /**
     * Записать по смещению
     */
    public function offsetSet(mixed $offset, mixed $value): void
    {
        if ($offset == 'settings') {
            !isset($this->settings) ? $this->settings = $value : throw new Exception('Запрещено перезаписывать настройки');
        } else if (isset($this->settings)) {
            $this->settings[$offset] = $value;
        } else {
            throw new Exception('Настройки не инициализированы');
        }
    }

    /**
     * Прочитать по смещению
     */
    public function offsetGet(mixed $offset): mixed
    {
        if ($offset == 'settings' && isset($this->settings)) {
            return $this->settings;
        } else if (isset($this->settings)) {
            if (isset($this->settings[$offset])) {
                return $this->settings[$offset];
            } else {
                throw new Exception('Не найдено: settings[\'' . $offset . '\']');
            }
        } else {
            throw new Exception('Настройки не инициализированы');
        }
    }

    /**
     * Проверка существования смещения
     */
    public function offsetExists(mixed $offset): bool
    {
        if ($offset == 'settings' && isset($this->settings)) {
            return isset($this->settings);
        } else if (isset($this->settings)) {
            return isset($this->settings[$offset]);
        } else {
            throw new Exception('Настройки не инициализированы');
        }
    }

    /**
     * Удалить по смещению
     */
    public function offsetUnset(mixed $offset): void
    {
        if ($offset == 'settings' && isset($this->settings)) {
            unset($this->settings);
        } else if (isset($this->settings)) {
            unset($this->settings[$offset]);
        } else {
            throw new Exception('Настройки не инициализированы');
        }
    }
}
