<?php

declare(strict_types=1);

namespace hood\vk\api;

use Exception;
use hood\vk\core,
    hood\vk\robots\robot;

/**
 * LongPoll
 *
 * $key Ключ к серверу
 * $server Сервер
 * $ts Идентификатор последнего события
 *
 * public function __construct(object $robot) Инициализация
 * public function get(int $wait = 25) Получить события
 * public function handle(callable $function, int $wait = 25) Обработать события
 *
 * @see https://vk.com/dev/bots_longpoll
 * @see https://vk.com/dev/groups.getLongPollServer
 * @see https://vk.com/dev/groups.setLongPollSettings
 *
 * @package hood\vk\api
 * @author Arsen Mirzaev Tatyano-Muradovich <red@hood.su>
 *
 * @todo Добавить обработку ошибок ($request['errors];)
 */
final class longpoll
{
    /**
     * Ключ к серверу
     *
     * @see $this->get()
     */
    private string $key;

    /**
     * Сервер (URL)
     *
     * @see $this->get()
     */
    private string $server;

    /**
     * Идентификатор последнего события
     *
     * От него отсчитываются новые, необработанные события
     *
     * @see $this->get()
     */
    private string $ts;

    /**
     * Инициализация
     *
     * $robot Робот
     */
    public function __construct(private robot $robot)
    {
        // Инициализация робота
        match (true) {
            !isset($robot->id) => throw new Exception('Необходимо указать идентификатор ВКонтакте'),
            !isset($robot->key) => throw new Exception('Необходимо указать ключ для доступа к LongPoll'),
            !isset($robot->version) => throw new Exception('Необходимо указать версию используемого API ВКонтакте'),
            default => null
        };

        // Остановка процессов-дубликатов
        if (!file_exists(core::init()->path_temp)) {
            // Если не существует каталога temp, то создать
            mkdir(core::init()->path_temp, 0775, true);
        }
        if (file_exists($lock = core::init()->path_temp . '/' . $this->robot->id . '_' . (int) $this->robot->session . '.longpoll')) {
            // Если существует файл-блокировщик, то удалить его
            unlink($lock);
        }
    }

    /**
     * Установить настройки
     *
     * Полная настройка и активация LongPoll
     *
     * $status = true Активация или деактивация
     * ...$params Изменяемые параметры
     */
    public function post(bool $status = true, string ...$params): array
    {
        // Инициализация настроек
        $settings = [
            'group_id'      => $this->robot->id,
            'access_token'  => $this->robot->key,
            'v'             => $this->robot->version,
            'api_version'   => $this->robot->version
        ];

        if ($status === true && !array_key_exists('enabled', $settings)) {
            // Если запущена активация и не был передан параметр статуса LongPoll

            // Установка параметра активации LongPoll
            $settings['enabled'] = 1;
        }

        // Установка переданных параметров
        foreach ($params as $param) {
            if ($param === 'group_id' || $param === 'access_token' || $param === 'v' || $param === 'api_version') {
                // Блокировка параметров от изменения
                continue;
            }

            // Конвертация типа
            $status = (int) $status;

            if ($param !== 'all') {
                // Если параметр не указывает на установку всех значений

                // Установка значения
                $settings[$param]                       = $status;
            } else {
                // Иначе установить все значения

                // Если передан параметр: установка ВСЕХ значений
                $settings['message_new']                = $status;
                $settings['message_reply']              = $status;
                $settings['message_allow']              = $status;
                $settings['message_deny']               = $status;
                $settings['message_edit']               = $status;
                $settings['message_typing_state']       = $status;
                $settings['photo_new']                  = $status;
                $settings['audio_new']                  = $status;
                $settings['video_new']                  = $status;
                $settings['wall_reply_new']             = $status;
                $settings['wall_reply_edit']            = $status;
                $settings['wall_reply_delete']          = $status;
                $settings['wall_reply_restore']         = $status;
                $settings['wall_post_new']              = $status;
                $settings['wall_repost']                = $status;
                $settings['board_post_new']             = $status;
                $settings['board_post_edit']            = $status;
                $settings['board_post_restore']         = $status;
                $settings['board_post_delete']          = $status;
                $settings['photo_comment_new']          = $status;
                $settings['photo_comment_edit']         = $status;
                $settings['photo_comment_delete']       = $status;
                $settings['photo_comment_restore']      = $status;
                $settings['video_comment_new']          = $status;
                $settings['video_comment_edit']         = $status;
                $settings['video_comment_delete']       = $status;
                $settings['video_comment_restore']      = $status;
                $settings['market_comment_new']         = $status;
                $settings['market_comment_edit']        = $status;
                $settings['market_comment_delete']      = $status;
                $settings['market_comment_restore']     = $status;
                $settings['poll_vote_new']              = $status;
                $settings['group_join']                 = $status;
                $settings['group_leave']                = $status;
                $settings['group_change_settings']      = $status;
                $settings['group_change_photo']         = $status;
                $settings['group_officers_edit']        = $status;
                $settings['user_block']                 = $status;
                $settings['user_unblock']               = $status;
                $settings['like_add']                   = $status;
                $settings['like_remove']                = $status;
                $settings['message_event']              = $status;
            }
        }

        return $this->robot->browser->request(method: 'POST', uri: 'groups.setLongPollSettings', options: $settings);
    }

    /**
     * Получить события
     *
     * $wait Время ожидания новых событий (в секундах)
     */
    public function get(int $wait = 25): array
    {
        if (empty($this->key) || empty($this->server) || empty($this->ts)) {
            // Если не инициализирован LongPoll-сервер

            // Запрос на получение доступа и данных LongPoll-сервера
            $response = json_decode($this->robot->browser->request(method: 'POST', uri: 'groups.getLongPollServer', options: [
                'form_params' => [
                    'group_id'      => $this->robot->id,
                    'v'             => $this->robot->version,
                    'access_token'  => $this->robot->key
                ],

            ])->getBody()->getContents())->response;

            // Ключ доступа
            $this->key      = $response->key;

            // Сервер хранящий события
            $this->server   = $response->server;

            // Идентификатор последнего события
            $this->ts       = $response->ts;
        }

        // Запрос на получение событий
        return json_decode($this->robot->browser->post($this->server . '?act=a_check&key=' . $this->key . '&ts=' . $this->ts . '&wait=' . $wait)->getBody()->getContents(), true);
    }

    /**
     * Обработать события
     *
     * Получает и обрабатывает события
     *
     * $function Обработка
     * $wait Время ожидания новых событий (в секундах)
     */
    public function handle(callable $function, int $wait = 25): array
    {
        // Файл-блокировщик и PID процесса
        $lock = core::init()->path_temp . '/' . $this->robot->id . '_' . (int) $this->robot->session . '.longpoll';
        $pid  = getmypid();

        // Создание или пересоздание файла-блокировщика
        file_put_contents($lock, $pid);

        do {
            // Выполняется пока существует файл-блокировщик

            // Запрос на получение событий
            $request = $this->get($wait);

            // [ВНИМАНИЕ] Соединение будет открыто даже при создании нового процесса LongPoll

            if (!file_exists($lock) || (int) fread(fopen($lock, 'r'), filesize($lock)) !== $pid) {
                // Проверка существования файла-блокировщика и соответствие его PID

                // Завершение работы
                break;
            }

            if (!empty($request['updates'])) {
                // Если получены необработанные события

                foreach ($request['updates'] as $update) {
                    // Перебор полученных событий
                    $function($update);
                }

                // Обновление идентификатора последнего события
                $this->ts = $request['ts'];
            }
        } while (true);

        return $request;
    }

    public function __destruct()
    {
        if (file_exists($lock = core::init()->path_temp . '/' . $this->robot->id . '_' . (int) $this->robot->session . '.longpoll')) {
            // Если существует файл-блокировщик, то удалить его
            unlink($lock);
        }
    }
}
